==> flame_behaviors-0.1.0.tar.gz: lib/src/behaviors/propagating_collision_behavior.dart <==
import 'package:flame/collisions.dart';
import 'package:flame/components.dart';
import 'package:flame_behaviors/flame_behaviors.dart';
import 'package:flutter/material.dart';

/// {@template collision_behavior}
/// This behavior is used for collision between entities. The
/// [PropagatingCollisionBehavior] propagates the collision to this behavior if
/// the entity that is colliding with the [Parent] is an instance of [Collider].
/// {@endtemplate}
abstract class CollisionBehavior<Collider extends Component,
    Parent extends Entity> extends Behavior<Parent> {
  /// Check if the given component is an instance of [Collider].
  bool isValid(Component c) => c is Collider;

  /// Called when the entity collides with [Collider].
  void onCollision(Set<Vector2> intersectionPoints, Collider other) {}

  /// Called when the entity starts to collides with [Collider].
  void onCollisionStart(Set<Vector2> intersectionPoints, Collider other) {}

  /// Called when the entity stops to collides with [Collider].
  void onCollisionEnd(Collider other) {}

  /// Whether the object is currently colliding with another [Collider] or not.
  bool get isColliding {
    final propagatingCollisionBehavior =
        parent.findBehavior<PropagatingCollisionBehavior>();
    final activeCollisions = propagatingCollisionBehavior.activeCollisions;

    return activeCollisions
        .map(propagatingCollisionBehavior._findEntity)
        .whereType<Collider>()
        .isNotEmpty;
  }
}

/// {@template propagating_collision_behavior}
/// This behavior is used to handle collisions between entities and propagates
/// the collision through to any [CollisionBehavior]s that are attached to the
/// entity.
///
/// The [CollisionBehavior]s are filtered by the [CollisionBehavior.isValid]
/// method by checking if the colliding entity is valid for the given behavior
/// and if the colliding entity is valid the [CollisionBehavior.onCollision] is
/// called.
///
/// This allows for strongly typed collision detection. Without having to add
/// multiple collision behaviors for different types of entities or adding more
/// logic to a single collision detection behavior.
///
/// If you have an entity that does not require any [CollisionBehavior]s of its
/// own, you can just add the hitbox directly to the entity's children.
/// Any other entity that has a [CollisionBehavior] for that entity attached
/// will then be able to collide with it.
///
/// **Note**: This behavior can only be used for collisions between entities.
/// It cannot be used for collisions between an entity and a non-entity
/// component.
/// {@endtemplate}
class PropagatingCollisionBehavior extends Behavior with CollisionCallbacks {
  /// {@macro propagating_collision_behavior}
  PropagatingCollisionBehavior(this._hitbox) : super(children: [_hitbox]);

  final ShapeHitbox _hitbox;

  @override
  Future<void> onLoad() async {
    _hitbox
      ..onCollisionCallback = onCollision
      ..onCollisionStartCallback = onCollisionStart
      ..onCollisionEndCallback = onCollisionEnd;
    parent.children.register<CollisionBehavior>();
    _propagateToBehaviors = parent.children.query<CollisionBehavior>();
  }

  /// List of [CollisionBehavior]s to which it can propagate to.
  List<CollisionBehavior> _propagateToBehaviors = [];

  Entity? _findEntity(PositionComponent other) {
    final parent = other.parent;
    if (parent is! PropagatingCollisionBehavior && parent is! Entity) {
      return null;
    }

    return parent is Entity
        ? parent
        : (parent as PropagatingCollisionBehavior?)!.parent;
  }

  @override
  void onCollisionStart(
    Set<Vector2> intersectionPoints,
    PositionComponent other,
  ) {
    activeCollisions.add(other);
    final otherEntity = _findEntity(other);
    if (otherEntity == null) {
      return super.onCollisionStart(intersectionPoints, other);
    }

    for (final behavior in _propagateToBehaviors) {
      if (behavior.isValid(otherEntity)) {
        behavior.onCollisionStart(intersectionPoints, otherEntity);
      }
    }
    return super.onCollisionStart(intersectionPoints, other);
  }

  @override
  @mustCallSuper
  void onCollision(Set<Vector2> intersectionPoints, PositionComponent other) {
    final otherEntity = _findEntity(other);
    if (otherEntity == null) {
      return super.onCollision(intersectionPoints, other);
    }

    for (final behavior in _propagateToBehaviors) {
      if (behavior.isValid(otherEntity)) {
        behavior.onCollision(intersectionPoints, otherEntity);
      }
    }
    super.onCollision(intersectionPoints, other);
  }

  @override
  void onCollisionEnd(PositionComponent other) {
    activeCollisions.remove(other);
    final otherEntity = _findEntity(other);
    if (otherEntity == null) {
      return super.onCollisionEnd(other);
    }

    for (final behavior in _propagateToBehaviors) {
      if (behavior.isValid(otherEntity)) {
        behavior.onCollisionEnd(otherEntity);
      }
    }
    return super.onCollisionEnd(other);
  }
}

==> flame_behaviors-0.1.1.tar.gz: lib/src/behaviors/propagating_collision_behavior.dart <==
import 'package:flame/collisions.dart';
import 'package:flame/components.dart';
import 'package:flame_behaviors/flame_behaviors.dart';
import 'package:flutter/material.dart';

/// {@template collision_behavior}
/// This behavior is used for collision between entities. The
/// [PropagatingCollisionBehavior] propagates the collision to this behavior if
/// the entity that is colliding with the [Parent] is an instance of [Collider].
/// {@endtemplate}
abstract class CollisionBehavior<Collider extends Component,
    Parent extends Entity> extends Behavior<Parent> {
  /// Check if the given component is an instance of [Collider].
  bool isValid(Component c) => c is Collider;

  /// Called when the entity collides with [Collider].
  void onCollision(Set<Vector2> intersectionPoints, Collider other) {}

  /// Called when the entity starts to collides with [Collider].
  void onCollisionStart(Set<Vector2> intersectionPoints, Collider other) {}

  /// Called when the entity stops to collides with [Collider].
  void onCollisionEnd(Collider other) {}

  /// Whether the object is currently colliding with another [Collider] or not.
  bool get isColliding {
    final propagatingCollisionBehavior =
        parent.findBehavior<PropagatingCollisionBehavior>();

    return propagatingCollisionBehavior.activeCollisions
        .map(propagatingCollisionBehavior._findEntity)
        .whereType<Collider>()
        .isNotEmpty;
  }
}

/// {@template propagating_collision_behavior}
/// This behavior is used to handle collisions between entities and propagates
/// the collision through to any [CollisionBehavior]s that are attached to the
/// entity.
///
/// The [CollisionBehavior]s are filtered by the [CollisionBehavior.isValid]
/// method by checking if the colliding entity is valid for the given behavior
/// and if the colliding entity is valid the [CollisionBehavior.onCollision] is
/// called.
///
/// This allows for strongly typed collision detection. Without having to add
/// multiple collision behaviors for different types of entities or adding more
/// logic to a single collision detection behavior.
///
/// If you have an entity that does not require any [CollisionBehavior]s of its
/// own, you can just add the hitbox directly to the entity's children.
/// Any other entity that has a [CollisionBehavior] for that entity attached
/// will then be able to collide with it.
///
/// **Note**: This behavior can also be used for collisions between entities
/// and non-entity components, by passing the component's type as the
/// `Collider` to the [CollisionBehavior].
/// {@endtemplate}
class PropagatingCollisionBehavior extends Behavior with CollisionCallbacks {
  /// {@macro propagating_collision_behavior}
  PropagatingCollisionBehavior(this._hitbox) : super(children: [_hitbox]);

  final ShapeHitbox _hitbox;

  @override
  Future<void> onLoad() async {
    _hitbox
      ..onCollisionCallback = onCollision
      ..onCollisionStartCallback = onCollisionStart
      ..onCollisionEndCallback = onCollisionEnd;
    parent.children.register<CollisionBehavior>();
    _propagateToBehaviors = parent.children.query<CollisionBehavior>();
  }

  /// List of [CollisionBehavior]s to which it can propagate to.
  List<CollisionBehavior> _propagateToBehaviors = [];

  /// Tries to find the entity that is colliding with the given entity.
  ///
  /// It will check if the parent is either a [PropagatingCollisionBehavior]
  /// or a [Entity]. If it is neither, it will return [other].
  Component? _findEntity(PositionComponent other) {
    final parent = other.parent;
    if (parent is! PropagatingCollisionBehavior && parent is! Entity) {
      if (other is ShapeHitbox) {
        return other.parent;
      }
      return other;
    }

    return parent is Entity
        ? parent
        : (parent as PropagatingCollisionBehavior?)!.parent;
  }

  @override
  void onCollisionStart(
    Set<Vector2> intersectionPoints,
    PositionComponent other,
  ) {
    activeCollisions.add(other);
    final otherEntity = _findEntity(other);
    if (otherEntity == null) {
      return;
    }

    for (final behavior in _propagateToBehaviors) {
      if (behavior.isValid(otherEntity)) {
        behavior.onCollisionStart(intersectionPoints, otherEntity);
      }
    }
    return super.onCollisionStart(intersectionPoints, other);
  }

  @override
  @mustCallSuper
  void onCollision(Set<Vector2> intersectionPoints, PositionComponent other) {
    final otherEntity = _findEntity(other);
    if (otherEntity == null) {
      return;
    }

    for (final behavior in _propagateToBehaviors) {
      if (behavior.isValid(otherEntity)) {
        behavior.onCollision(intersectionPoints, otherEntity);
      }
    }
    super.onCollision(intersectionPoints, other);
  }

  @override
  void onCollisionEnd(PositionComponent other) {
    activeCollisions.remove(other);
    final otherEntity = _findEntity(other);
    if (otherEntity == null) {
      return;
    }

    for (final behavior in _propagateToBehaviors) {
      if (behavior.isValid(otherEntity)) {
        behavior.onCollisionEnd(otherEntity);
      }
    }
    return super.onCollisionEnd(other);
  }
}
